#!/usr/bin/env python3

import subprocess
import numpy as np
import matplotlib.pyplot as plt


SETPOINT = 2    # deg
TMAX = 10       # sec
MATPLOTLIB_FIGSIZE = (2*6.4, 2*4.8)


def rads2degs(rs):
    return [r*180/np.pi for r in rs]


#
# Load
#

def parse_raw(raw):
    *data, result = raw.strip().split('\n')
    tau = [float(f) for f in result.strip().split(';')][2]
    parsed = [[float(f) for f in line.strip().split(';')] for line in data]
    ts = [tup[0] for tup in parsed]
    poss = [tup[1] for tup in parsed]
    vels = [tup[2] for tup in parsed]
    accs = [tup[3] for tup in parsed]
    return tau, ts, poss, vels, accs

raw1 = subprocess.check_output(
    ['cargo', 'run', '--', 'one', '2', '-0.5'], cwd="pid-sim"
).decode('utf8')
raw2 = subprocess.check_output(
    ['cargo', 'run', '--', 'one', '12.150', '-3.731'], cwd="pid-sim"
).decode('utf8')

tau1, ts1, poss1, vels1, accs1 = parse_raw(raw1)
tau2, ts2, poss2, vels2, accs2 = parse_raw(raw2)

print("1: tau=%.3f" % tau1)
print("2: tau=%.3f" % tau2)

#
# Show
#

def configure_subplot(subplot_idx, ylabel):
    plt.subplot(3, 1, subplot_idx)
    plt.ylabel(ylabel)
    plt.xlim((0, TMAX))
    plt.axhline(0, color="gray")


plt.figure(figsize=MATPLOTLIB_FIGSIZE)

configure_subplot(1, "pos (deg)")
plt.axhline(SETPOINT, color="gray")
plt.axhline(SETPOINT*0.90, linestyle=':', color="gray")
plt.axhline(SETPOINT*1.10, linestyle=':', color="gray")
plt.axvline(tau1, color="darkblue")
plt.axvline(tau2, color="brown")
h1, = plt.plot(ts1, rads2degs(poss1))
h2, = plt.plot(ts2, rads2degs(poss2))
plt.legend([h1, h2], ["Kp=2, Kd=-0.5", "Kp=12.150, Kd=-3.731"], loc="upper right")

configure_subplot(2, "vel (deg/sec)")
plt.plot(ts1, rads2degs(vels1))
plt.plot(ts2, rads2degs(vels2))

configure_subplot(3, "acc (deg/sec**2)")
plt.plot(ts1, rads2degs(accs1))
plt.plot(ts2, rads2degs(accs2))
plt.xlabel("t (sec)")

plt.tight_layout()
plt.savefig("1-step-response.png")

plt.show()
